@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2013 Codership Oy <info@codership.com>
+// Copyright (C) 2013-2018 Codership Oy <info@codership.com>
 //
 
 
@@ -43,15 +43,18 @@ public:
 
     static Version version (const std::string& ver);
 
+    static const char* type(wsrep_key_type_t const t);
+
     class Key
     {
     public:
-        enum Prefix
+        enum Prefix // this stays for backward compatibility
         {
             P_SHARED = 0,
-            P_EXCLUSIVE,
-            P_LAST = P_EXCLUSIVE
+            P_EXCLUSIVE
         };
+
+        static int const TYPE_MAX = WSREP_KEY_EXCLUSIVE;
     }; /* class Key */
 
     /* This class describes what commonly would be referred to as a "key".
@@ -73,9 +76,9 @@ public:
          * from a key hash and optional annotation. */
         KeyPart (TmpStore&       tmp,
                  const HashData& hash,
-                 Version const   ver,
-                 bool const      exclusive,
                  const wsrep_buf_t* parts, /* for annotation */
+                 Version const   ver,
+                 int const       prefix,
                  int const       part_num,
                  int const       alignment
             )
@@ -99,7 +102,8 @@ public:
             gu::byte_t b = tmp.buf[0] & (~HEADER_MASK);
 
             /* set prefix  */
-            if (exclusive) { b |= (Key::P_EXCLUSIVE & PREFIX_MASK); }
+            assert(prefix <= PREFIX_MASK);
+            b |= (prefix & PREFIX_MASK);
 
             /* set version */
             b |= (ver & VERSION_MASK) << PREFIX_BITS;
@@ -126,19 +130,58 @@ public:
 
         explicit KeyPart (const gu::byte_t* ptr = NULL) : data_(ptr) {}
 
-        Key::Prefix prefix() const
+        /* converts wsrep key type to KeyPart "prefix" depending on writeset
+         * version */
+        static int prefix(wsrep_key_type_t const ws_type, int const ws_ver)
         {
-            gu::byte_t const p(data_[0] & PREFIX_MASK);
-
-            if (gu_likely(p <= Key::P_LAST))
-                return static_cast<Key::Prefix>(p);
-
-            throw_bad_prefix(p);
+            if (ws_ver >= 0 && ws_ver <= 4)
+            {
+                switch (ws_type)
+                {
+                case WSREP_KEY_SHARED:
+                    return 0;
+                case WSREP_KEY_SEMI:
+                    return 1;
+                case WSREP_KEY_EXCLUSIVE:
+                    return ws_ver < 4 ? KeySet::Key::P_EXCLUSIVE : 2;
+                }
+            }
+            assert(0);
+            throw_bad_type_version(ws_type, ws_ver);
         }
 
-        bool shared()       const { return prefix() == Key::P_SHARED; }
+        /* The return value is subject to interpretation based on the
+         * writeset version which is done in wsrep_type(int) method */
+        int prefix() const
+        {
+            return (data_[0] & PREFIX_MASK);
+        }
 
-        bool exclusive()    const { return prefix() == Key::P_EXCLUSIVE; }
+        wsrep_key_type_t wsrep_type(int const ws_ver) const
+        {
+            assert(ws_ver >= 0 && ws_ver <= 4);
+
+            wsrep_key_type_t ret;
+
+            switch (prefix())
+            {
+            case 0:
+                ret = WSREP_KEY_SHARED;
+                break;
+            case 1:
+                ret = ws_ver == 4 ? WSREP_KEY_SEMI : WSREP_KEY_EXCLUSIVE;
+                break;
+            case 2:
+                assert(ws_ver == 4);
+                ret = WSREP_KEY_EXCLUSIVE;
+                break;
+            default:
+                throw_bad_prefix(prefix());
+            }
+
+            assert(prefix() == prefix(ret, ws_ver));
+            return ret;
+        }
 
         static Version version(const gu::byte_t* const buf)
         {
@@ -311,6 +354,8 @@ public:
         static void
         throw_buffer_too_short (size_t expected, size_t got) GU_NORETURN;
         static void
+        throw_bad_type_version (wsrep_key_type_t t, int v)   GU_NORETURN;
+        static void
         throw_bad_prefix       (gu::byte_t p)                GU_NORETURN;
         static void
         throw_match_empty_key  (Version my, Version other)   GU_NORETURN;
@@ -535,6 +580,7 @@ public:
                  const KeyPart* parent,
                  const KeyData& kd,
                  int const      part_num,
+                 int const      ws_ver,
                  int const      alignment);
 
         KeyPart (const KeyPart& k)
@@ -572,11 +618,8 @@ public:
             return (size_ == s && !(::memcmp (value_, v, size_)));
         }
 
-        bool
-        exclusive () const { return (part_ && part_->exclusive()); }
-
-        bool
-        shared () const { return !exclusive(); }
+        int
+        prefix() const { return (part_ ? part_->prefix() : 0); }
 
         void
         acquire()
@@ -631,7 +674,8 @@ public:
                size_t                  reserved_size,
                const BaseName&         base_name,
                KeySet::Version const   version,
-               gu::RecordSet::Version const rsv)
+               gu::RecordSet::Version const rsv,
+               int const               ws_ver)
         :
         gu::RecordSetOut<KeySet::KeyPart> (
             reserved,
@@ -643,10 +687,12 @@ public:
         added_(),
         prev_ (),
         new_  (),
-        version_(version)
+        version_(version),
+        ws_ver_(ws_ver)
     {
         assert (version_ != KeySet::EMPTY);
         assert ((uintptr_t(reserved) % GU_WORD_BYTES) == 0);
+        assert (ws_ver <= 4);
         KeyPart zero(version_);
         prev_().push_back(zero);
     }
@@ -666,6 +712,7 @@ private:
     gu::Vector<KeyPart,5> prev_;
     gu::Vector<KeyPart,5> new_;
     KeySet::Version       version_;
+    int                   ws_ver_;
 
     static gu::RecordSet::CheckType
     check_type (KeySet::Version ver)
