@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2018 Codership Oy <info@codership.com>
 //
 
 //! @file replicator_smm.hpp
@@ -88,11 +88,6 @@ namespace galera
             wsdb_.discard_conn_query(conn_id);
         }
 
-        void discard_local_conn(wsrep_conn_id_t conn_id)
-        {
-            wsdb_.discard_conn(conn_id);
-        }
-
         void apply_trx(void* recv_ctx, TrxHandle* trx);
 
         wsrep_status_t replicate(TrxHandle* trx, wsrep_trx_meta_t*);
@@ -487,17 +482,19 @@ namespace galera
 
         static int const       MAX_PROTO_VER;
         /*
-         * |------------------------------------------------------
-         * | protocol_version_ |  trx  version  | str_proto_ver_ |
-         * |------------------------------------------------------
-         * |                 1 |              1 |              0 |
-         * |                 2 |              1 |              1 |
-         * |                 3 |              2 |              1 |
-         * |                 4 |              2 |              1 |
-         * |                 5 |              3 |              1 |
-         * |                 6 |              3 |              2 |
-         * |                 7 |              3 |              2 |
-         * -------------------------------------------------------
+         * |--------------------------------------------------------------------|
+         * | protocol_version_ | trx version | str_proto_ver_ | record_set_ver_ |
+         * |--------------------------------------------------------------------|
+         * |                 1 |           1 |              0 |               1 |
+         * |                 2 |           1 |              1 |               1 |
+         * |                 3 |           2 |              1 |               1 |
+         * |                 4 |           2 |              1 |               1 |
+         * |                 5 |           3 |              1 |               1 |
+         * |                 6 |           3 |              2 |               1 |
+         * |                 7 |           3 |              2 |               1 |
+         * |                 8 |           3 |              2 |               2 |
+         * |                 9 |           4 |              2 |               2 |
+         * |--------------------------------------------------------------------|
          */
 
         int                    str_proto_ver_;// state transfer request protocol
@@ -514,6 +511,10 @@ namespace galera
         std::string           state_file_;
         SavedState            st_;
 
+        // boolean telling if the node is safe to use for bootstrapping
+        // a new primary component
+        bool safe_to_bootstrap_;
+
         // currently installed trx parameters
         TrxHandle::Params     trx_params_;
 
