@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2016 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2018 Codership Oy <info@codership.com>
  */
 
 #ifndef __GCACHE_H__
@@ -192,11 +192,15 @@ namespace gcache
             size_t rb_size()             const { return rb_size_;         }
             size_t page_size()           const { return page_size_;       }
             size_t keep_pages_size()     const { return keep_pages_size_; }
+            int    debug()               const { return debug_;           }
             bool   recover()             const { return recover_;         }
 
             void mem_size        (size_t s) { mem_size_        = s; }
             void page_size       (size_t s) { page_size_       = s; }
             void keep_pages_size (size_t s) { keep_pages_size_ = s; }
+#ifndef NDEBUG
+            void debug           (int    d) { debug_           = d; }
+#endif
 
         private:
 
@@ -206,6 +210,7 @@ namespace gcache
             size_t      const rb_size_;
             size_t            page_size_;
             size_t            keep_pages_size_;
+            int               debug_;
             bool        const recover_;
         }
             params;
@@ -232,9 +237,14 @@ namespace gcache
         std::set<const void*> buf_tracker;
 #endif
 
+        void discard_buffer (BufferHeader* bh);
+
         /* returns true when successfully discards all seqnos up to s */
         bool discard_seqno (int64_t s);
 
+        /* discards all seqnos greater than s */
+        void discard_tail (int64_t s);
+
         // disable copying
         GCache (const GCache&);
         GCache& operator = (const GCache&);
