@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2016 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2018 Codership Oy <info@codership.com>
  */
 
 #include "gcache_bh.hpp"
@@ -23,10 +23,19 @@ namespace gcache
 
         assert(seqno2ptr.empty() || seqno_max == seqno2ptr.rbegin()->first);
 
-        if (g == gid && s == seqno_max) return;
+        if (g == gid && s != SEQNO_ILL && seqno_max >= s)
+        {
+            if (seqno_max > s)
+            {
+                discard_tail(s);
+                seqno_max = s;
+                seqno_released = s;
+            }
+            return;
+        }
 
-        log_info << "GCache history reset: old(" << gid << ':' << seqno_max
-                 << ") -> new(" << g << ':' << s << ")";
+        log_info << "GCache history reset: " << gid << ':' << seqno_max
+                 << " -> " << g << ':' << s;
 
         seqno_released = SEQNO_NONE;
         gid = g;
@@ -109,7 +118,7 @@ namespace gcache
 
             if (gu_unlikely(it == seqno2ptr.end()))
             {
-                /* This means that there are no element with
+                /* This means that there are no elements with
                  * seqno following seqno_released - and this should not
                  * generally happen. But it looks like stopcont test does it. */
                 if (SEQNO_NONE != seqno_released)
@@ -131,10 +140,13 @@ namespace gcache
             int64_t const start(it->first - 1);
             int64_t const end  (seqno - start >= 2*batch_size ?
                                 start + batch_size : seqno);
-#if 0
-            log_info << "############ releasing " << (seqno - start)
-                     << " buffers, batch_size: " << batch_size
-                     << ", end: " << end;
+#ifndef NDEBUG
+            if (params.debug())
+            {
+                log_info << "GCache::seqno_release(" << seqno << "): "
+                         << (seqno - start) << " buffers, batch_size: "
+                         << batch_size << ", end: " << end;
+            }
 #endif
             for (;(loop = (it != seqno2ptr.end())) && it->first <= end;)
             {
